/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <search.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <gdk/gdk.h>

#include "qelem.h"
#include "scribble-simple.h"
#include "points.h"
#include "manifold.h"
#include "myidle.h"
#include "field.h"

/* #define CP_TRACE 1 */

extern int step_flag;


/* display text here */
GtkText *text;
/* describe a font */
GdkFont *fixed_font;
/* Backing pixmap for drawing area */
GdkPixmap *pixmap = NULL;
/* drawing widget */
GtkWidget *drawing_area;

gint my_idle_idx;
/* forward references */

/* draw a point */
static void draw_brush( GtkWidget *widget, gdouble    x, gdouble    y);

/* start at time 0 */
unsigned long my_time = 0;

/* draw once */
int once_flag = 0;

/* Create a new backing pixmap of the appropriate size */
static gint configure_event( GtkWidget         *widget,
                             GdkEventConfigure *event )
{
  if (pixmap)
    g_object_unref (pixmap);

  pixmap = gdk_pixmap_new (widget->window,
			   widget->allocation.width,
			   widget->allocation.height,
			   -1);
  gdk_draw_rectangle (pixmap,
		      widget->style->white_gc,
		      TRUE,
		      0, 0,
		      widget->allocation.width,
		      widget->allocation.height);

  /* once_flag = 0; */

  return TRUE;
}

/* Redraw the screen from the backing pixmap */
static gint expose_event( GtkWidget      *widget,
                          GdkEventExpose *event )
{
  gdk_draw_drawable (widget->window,
		     widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		     pixmap,
		     event->area.x, event->area.y,
		     event->area.x, event->area.y,
		     event->area.width, event->area.height);


  /* once_flag = 0; */

  return FALSE;
}

/* Draw a rectangle on the screen */
static void draw_brush( GtkWidget *widget,
                        gdouble    x,
                        gdouble    y)
{
  GdkRectangle update_rect;

  update_rect.x = x - 1;
  update_rect.y = y - 1;
  update_rect.width = 1;
  update_rect.height = 1;
  gdk_draw_rectangle (pixmap,
		      widget->style->black_gc,
		      TRUE,
		      update_rect.x, update_rect.y,
		      update_rect.width, update_rect.height);
  gtk_widget_queue_draw_area (widget, 
		      update_rect.x, update_rect.y,
		      update_rect.width, update_rect.height);
}

static gint button_press_event( GtkWidget      *widget,
                                GdkEventButton *event )
{
  if (event->button == 1 && pixmap != NULL) {
    char wbuf[256];
    int xi,yi;

    xi = event->x;
    yi = event->y;

    sprintf(wbuf,"man[%3d,%3d] = %f, %f\n",
	    xi, yi,
	    man[xi][yi].x, man[xi][yi].y );
    gtk_text_insert (text, fixed_font, NULL, NULL,
		     wbuf, -1
		     );

    {
      double fx,fy;
      POINT p;
      int i;

      for ( i = 0 ; i < pts_idx ; i++ ) {
	/* get a point */
	memcpy(&p, &pts[i], sizeof(POINT));

	/* pretend its at mouse click location */
	p.x = (double)xi + 0.5;
	p.y = (double)yi + 0.5;

	/* get forces at point 'p' from all fields */
	get_all_forces ( &p, &fx, &fy, 0 );

	sprintf(wbuf,"Point: %d at [%d,%d] = %f %f\n",
		i+1,
		xi,yi,
		fx,fy);
	gtk_text_insert (text, fixed_font, NULL, NULL, wbuf, -1 );
      }

    } /* end of block */

#if 0
    draw_brush (widget, event->x, event->y);
#endif
  }

  return TRUE;
}

static gint motion_notify_event( GtkWidget *widget,
                                 GdkEventMotion *event )
{
  int x, y;
  GdkModifierType state;

  if (event->is_hint)
    gdk_window_get_pointer (event->window, &x, &y, &state);
  else
    {
      x = event->x;
      y = event->y;
      state = event->state;
    }
    
  if (state & GDK_BUTTON1_MASK && pixmap != NULL)
    draw_brush (widget, x, y);
  
  return TRUE;
}

void quit ()
{
  exit (0);
}

extern double cur_div;
extern double cur_div_inc;

GtkWidget *hbox_label;
GtkWidget *label [ 4 ];

/* draw an arc */
int draw_arc ( );
/* inc */
void do_inc();
/* dec */
void do_dec();

/* get_inc */
void get_inc ()
{
  char *r;

  cur_div     = strtod ( gtk_entry_get_text( GTK_ENTRY ( label [ 1 ] )), &r);  
  cur_div_inc = strtod ( gtk_entry_get_text( GTK_ENTRY ( label [ 3 ] )), &r);
}

/* show aspect */
void show_aspect()
{
  char wbuf [ 80 ];

  sprintf(wbuf,"%lf",cur_div);
  gtk_entry_set_text ( GTK_ENTRY(label [ 1 ]), wbuf);
}

extern int run_flag;

/* run */
void run_click ()
{
  step_flag = 0;
  get_inc();
  run_flag = 1;
  my_idle_idx = gtk_idle_add ( my_idle, NULL );
}

extern int grid_flag;

/* grid toggle */
void grid_toggle_click ()
{
  grid_flag ^= 1;
  draw_arc();
}

extern int circ_flag;

/* circ toggle click */
void circ_toggle_click ()
{
  circ_flag ^= 1;
  draw_arc();
}

/* stop */
void stop_click()
{
  run_flag = 0;
  show_aspect();
}

/* user clicked forward */
void for_click ()
{
  step_flag = 1;
  my_idle_idx = gtk_idle_add ( my_idle, NULL );
}

/* user clicked reverse */
void rev_click ()
{
  step_flag = -1;
  my_idle_idx = gtk_idle_add ( my_idle, NULL );
}

  GtkWidget *window;

int main( int   argc, char *argv[] )
{
  GtkWidget *vbox;
  GtkWidget *hbox;

  GtkWidget *button;

  GtkWidget *button_vbox;
  GtkWidget *buttons [ 10 ];

  gtk_init (&argc, &argv);

  /* Load a fixed font */
  fixed_font = gdk_font_load ("-adobe-times-*-*-*-*-*-140-*-*-*-*-*-*");

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "Inertia");

  gtk_widget_set_name (window, "Test Input");

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), hbox);
  gtk_widget_show ( hbox );

  {
    /*
     * buttons
     */
    button_vbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start ( GTK_BOX(hbox), button_vbox, 0,0,0 );
    gtk_widget_show (button_vbox);

    /* .. Run */
    buttons[0] = gtk_button_new_with_label ("Run");
    gtk_widget_show ( buttons[0] );
    gtk_box_pack_start (GTK_BOX (button_vbox), buttons[0], FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (buttons[0]), "clicked",
		      G_CALLBACK (run_click), NULL);

    /* .. For */
    buttons[1] = gtk_button_new_with_label ("For");
    gtk_widget_show ( buttons[1] );
    gtk_box_pack_start (GTK_BOX (button_vbox), buttons[1], FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (buttons[1]), "clicked",
		      G_CALLBACK (for_click), NULL);

    /* .. Rev */
    buttons[2] = gtk_button_new_with_label ("Rev");
    gtk_widget_show ( buttons[2] );
    gtk_box_pack_start (GTK_BOX (button_vbox), buttons[2], FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (buttons[2]), "clicked",
		      G_CALLBACK (rev_click), NULL);

    /* .. Stop */
    buttons[3] = gtk_button_new_with_label ("Stop");
    gtk_widget_show ( buttons[3] );
    gtk_box_pack_start (GTK_BOX (button_vbox), buttons[3], FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (buttons[3]), "clicked",
		      G_CALLBACK (stop_click), NULL);

    /* .. Grid */
    buttons[4] = gtk_button_new_with_label ("Grid");
    gtk_widget_show ( buttons[4] );
    gtk_box_pack_start (GTK_BOX (button_vbox), buttons[4], FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (buttons[4]), "clicked",
		      G_CALLBACK (grid_toggle_click), NULL);

#if 0
    /* .. UnCirc */
    buttons[5] = gtk_button_new_with_label ("UnAtom");
    gtk_widget_show ( buttons[5] );
    gtk_box_pack_start (GTK_BOX (button_vbox), buttons[5], FALSE, FALSE, 0);
    g_signal_connect (G_OBJECT (buttons[5]), "clicked",
		      G_CALLBACK (circ_toggle_click), NULL);
#endif

  }

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start ( GTK_BOX(hbox), vbox, 0,0,0 );
  gtk_widget_show (vbox);

  g_signal_connect (G_OBJECT (window), "destroy",
                    G_CALLBACK (quit), NULL);

  /* Create the drawing area */

  drawing_area = gtk_drawing_area_new ();
  gtk_widget_set_size_request (GTK_WIDGET (drawing_area), WIN_SIZE_X, WIN_SIZE_Y);
  gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);
  gtk_widget_show (drawing_area);

  /* Signals used to handle backing pixmap */

  g_signal_connect (G_OBJECT (drawing_area), "expose_event",
		    G_CALLBACK (expose_event), NULL);
  g_signal_connect (G_OBJECT (drawing_area),"configure_event",
		    G_CALLBACK (configure_event), NULL);

  /* Event signals */

  g_signal_connect (G_OBJECT (drawing_area), "motion_notify_event",
		    G_CALLBACK (motion_notify_event), NULL);
  g_signal_connect (G_OBJECT (drawing_area), "button_press_event",
		    G_CALLBACK (button_press_event), NULL);

  gtk_widget_set_events (drawing_area, GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_POINTER_MOTION_MASK
			 | GDK_POINTER_MOTION_HINT_MASK);

  /* .. And a text area */
  text = (GtkText *)gtk_text_new (NULL, NULL);
  gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET(text), FALSE, FALSE, 0);

  {
    /* display labels */
    char wbuf [ 80 ];

    hbox_label = gtk_hbox_new(TRUE, 0);
    gtk_widget_show(hbox_label);

    label [ 0 ] = gtk_label_new ( "Aspect" );
    gtk_widget_show( label [ 0 ] );
    gtk_box_pack_start (GTK_BOX (hbox_label), GTK_WIDGET( label[0] ), FALSE, FALSE, 0);

    sprintf(wbuf,"%lf",cur_div);
    label [ 1 ] = gtk_entry_new_with_max_length ( 12 );
    gtk_entry_set_text ( GTK_ENTRY(label [ 1 ]), wbuf);
    gtk_widget_show( label [ 1 ] );
    gtk_box_pack_start (GTK_BOX (hbox_label), GTK_WIDGET( label[1] ), FALSE, FALSE, 0);

    label [ 2 ] = gtk_label_new ( "Inc" );
    gtk_widget_show( label [ 2 ] );
    gtk_box_pack_start (GTK_BOX (hbox_label), GTK_WIDGET( label[2] ), FALSE, FALSE, 0);

    sprintf(wbuf,"%lf",cur_div_inc);
    label [ 3 ] = gtk_entry_new_with_max_length ( 12 );
    gtk_entry_set_text ( GTK_ENTRY(label [ 3 ]), wbuf);
    gtk_widget_show( label [ 3 ] );
    gtk_box_pack_start (GTK_BOX (hbox_label), GTK_WIDGET( label[3] ), FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (vbox), hbox_label, FALSE, FALSE, 0);
  }

  {
    /* .. And a quit button */
    button = gtk_button_new_with_label ("Quit");
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);

    g_signal_connect_swapped (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_widget_destroy),
			      window);
    gtk_widget_show (button);
    gtk_widget_show (window);
    gtk_widget_show (GTK_WIDGET(text));
  }

  /* add an idle routine */
  my_idle_idx = gtk_idle_add ( my_idle, NULL );

  /* off to main */
  gtk_main ();

  /* done */
  return 0;
}
